import pandas as pd


def _equal_amount_of_unknown_alleles(
    input_df_donor: pd.DataFrame, input_df_recipient: pd.DataFrame
) -> bool:
    """
    :param input_df_donor: Input pandas.DataFrame with the donor alleles
    :param input_df_recipient: Input pandas.DataFrame with the recipient alleles

    :return: True if the amount of unknown* alleles is equal in both dataframes, False otherwise

    Also tests if the unknown* alleles are either for both the alleles of the locus or for none of them.
    * unknown alleles are the ones that were inputted as "A*", "B*", "C*", "DRB1*", "DQA1*", "DQB1*", "DPA1*" and/or
    "DPB1*". Here we are not talking about the alleles that are unknown to the database.
    """

    unknown_alleles: list[str] = [
        "A*",
        "B*",
        "C*",
        "DRB1*",
        # No DRB345 because it is ok for them to have Nops
        "DQA1*",
        "DQB1*",
        "DPA1*",
        "DPB1*",
    ]

    # Check if the amount of unknown alleles is equal in both dataframes
    for (_, row_d), (_, row_r) in zip(
        input_df_donor.iterrows(), input_df_recipient.iterrows()
    ):
        allele: str
        for allele in unknown_alleles:
            locus: str = allele[:-1]
            locus_donor_1: str = f"{locus}1_D"
            locus_donor_2: str = f"{locus}2_D"
            locus_recipient_1: str = f"{locus}1_R"
            locus_recipient_2: str = f"{locus}2_R"

            if allele in [row_r[locus_recipient_1], row_d[locus_donor_1]]:
                # If the (first, otherwise it can be an homozygote) allele is unknown, then the other one must be
                # unknown too and the same for the other individual
                if not (
                    allele
                    ==
                    row_r[locus_recipient_1]
                    ==
                    row_r[locus_recipient_2]
                    ==
                    row_d[locus_donor_1]
                    ==
                    row_d[locus_donor_2]
                ):
                    return False

    return True



import os
import pandas as pd


def _open_epregistry_database(
        path_to_csv: str,
        ghost_alleles: list[str],
        no_eplets: bool = False
) -> pd.DataFrame:
    """
    :param path_to_csv: path to the csv EpRegistry database file
    :param ghost_alleles: allele(s) string(s) to be used as (a) ghost allele(s) (no eplets)

    :return: pandas.DataFrame with the EpRegistry database and the ghost allele
    """

    df_db: pd.DataFrame

    file_name_no_extension: str = path_to_csv.split('.csv')[0]
    suffix: str = ghost_alleles[0]
    pickle_file_name: str = f"{file_name_no_extension}_{no_eplets}_{suffix}.pickle"

    # replace * by _ in the pickle file name to avoid problems when * is in the file name
    pickle_file_name = pickle_file_name.replace('*', '_')

    # if pickle file exists, load it
    if os.path.exists(pickle_file_name):
        df_db = pd.read_pickle(pickle_file_name)
        return df_db
    # if pickle file does not exist, create it
    else:
        if no_eplets:
            df_db = pd.read_csv(path_to_csv, sep=";", usecols=[0]).set_index("allele")
            # add a row to an empty dataframe with concat
            for ghost_allele in ghost_alleles:
                df_db = pd.concat([df_db, pd.DataFrame(columns=[], index=[ghost_allele])])
        else:
            df_db = pd.read_csv(path_to_csv, sep=";").set_index("allele")
            for ghost_allele in ghost_alleles:
                # add row with nan values
                df_db.loc[ghost_allele] = float("nan")

        df_db.to_pickle(pickle_file_name)

    return df_db


def open_ep_data(eplet_comparison_file_directory_path: str) -> pd.DataFrame:
    """
    :param eplet_comparison_file_directory_path: path to where the data folder is located
    :return: pandas.DataFrame with the eplet informations database
    """

    df_ep_data: pd.DataFrame

    # if pickle file exists, load it
    if os.path.exists(f"{eplet_comparison_file_directory_path}/data/ep_data.pickle"):
        df_ep_data = pd.read_pickle(f"{eplet_comparison_file_directory_path}/data/ep_data.pickle")
        return df_ep_data
    # if pickle file does not exist, create it
    else:
        df_ep_data = pd.read_csv(f"{eplet_comparison_file_directory_path}/data/ep_data.csv", sep=";")
        df_ep_data.to_pickle(f"{eplet_comparison_file_directory_path}/data/ep_data.pickle")
        return df_ep_data
